export const myFetchContentType = {
  X_WWW_FORM_URLENCODED: 0,
  JSON: 1,
};

export function myFetch({
  url,
  method,
  body = null,
  headers = {},
  contentType = myFetchContentType.JSON,
  showNotification = null,
}) {
  const getContentType = () => {
    return contentType === myFetchContentType.JSON
      ? "application/json"
      : "application/x-www-form-urlencoded";
  };

  const getBody = () => {
    if (!body) return null;

    if (contentType === myFetchContentType.X_WWW_FORM_URLENCODED)
      return new URLSearchParams(body);

    if (contentType === myFetchContentType.JSON) return JSON.stringify(body);

    return body;
  };

  // abort fetch if it takes to long
  const controller = new AbortController();
  const signal = controller.signal;

  setTimeout(() => controller.abort(), 30000); // 30 seconds

  const requestOptions = {
    method: method,
    headers: {
      "Content-Type": getContentType(),
      ...headers,
    },
    body: getBody(),
    signal: signal,
  };

  return fetch(`${window.location.origin}${url}`, requestOptions)
    .then((response) => {
      // if status is not in range 200-299
      if (!response.ok) throw response.status;

      // check if response is json
      if (response.headers.get("content-type")?.includes("application/json")) {
        return response.json();
      }

      return response.text();
    })
    .catch((errorStatus) => {
      // ignore errors here as they are handled in the components
      if (errorStatus === 400) throw errorStatus;

      if (showNotification !== null) {
        if (errorStatus === 500) {
          showNotification("error", "Unerwarteter Fehler aufgetreten");
        }
      }

      throw errorStatus;
    });
}
